@@ -19,8 +19,8 @@
  * @file globus_ftp_client.c
  *
  * $RCSfile: globus_ftp_client.c,v $
- * $Revision: 1.18 $
- * $Date: 2008/04/04 01:51:46 $
+ * $Revision: 1.19 $
+ * $Date: 2010/03/24 04:50:35 $
  */
 #endif
 
@@ -149,9 +149,10 @@ globus_l_ftp_client_activate(void)
     globus_i_ftp_client_debug_printf(1,
         (stderr, "globus_l_ftp_client_activate() exiting\n"));
 
+    if(globus_i_ftp_client_find_ssh_client_program() != NULL)
     {
-        globus_result_t res;
-
+        globus_result_t             res;
+        
         res = globus_xio_driver_load("popen", &ftp_client_i_popen_driver);
         if(res == GLOBUS_SUCCESS)
         {
@@ -159,7 +160,6 @@ globus_l_ftp_client_activate(void)
             globus_xio_stack_push_driver(
                 ftp_client_i_popen_stack, ftp_client_i_popen_driver);
             ftp_client_i_popen_ready = GLOBUS_TRUE;
-            globus_i_ftp_client_find_ssh_client_program();
         }
     }
 
