@@ -392,6 +392,12 @@ globus_ftp_client_restart_marker_get_total(
     globus_ftp_client_restart_marker_t *	marker,
     globus_off_t *				total_bytes);
 
+globus_result_t
+globus_ftp_client_restart_marker_get_first_block(
+    globus_ftp_client_restart_marker_t *        marker,
+    globus_off_t *                              offset,
+    globus_off_t *                              end_offset);
+
 globus_result_t
 globus_ftp_client_restart_marker_to_string(
     globus_ftp_client_restart_marker_t *	marker,
@@ -446,6 +452,11 @@ globus_ftp_client_handle_get_user_pointer(
     const globus_ftp_client_handle_t *		handle,
     void **					user_pointer);
 
+globus_result_t
+globus_ftp_client_handle_get_restart_marker(
+    const globus_ftp_client_handle_t *          handle,
+    globus_ftp_client_restart_marker_t *        marker);
+
 globus_result_t
 globus_ftp_client_handle_add_plugin(
     globus_ftp_client_handle_t *		handle,
@@ -549,6 +560,20 @@ globus_ftp_client_handleattr_set_netlogger(
     globus_ftp_client_handleattr_t *		attr,
     globus_netlogger_handle_t *	                nl_handle);
 
+globus_result_t
+globus_ftp_client_handleattr_set_clientinfo(
+    globus_ftp_client_handleattr_t *    attr,
+    const char *                        app_name,
+    const char *                        app_version,
+    const char *                        other);
+
+globus_result_t
+globus_ftp_client_handleattr_get_clientinfo(
+    globus_ftp_client_handleattr_t *    attr,
+    char **                             app_name,
+    char **                             app_version,
+    char **                             other);
+
 globus_result_t
 globus_ftp_client_handleattr_set_netlogger_ftp_io(
     globus_ftp_client_handleattr_t *            attr,
@@ -1143,6 +1168,7 @@ typedef enum
     GLOBUS_FTP_CLIENT_FEATURE_CKSM,
     GLOBUS_FTP_CLIENT_FEATURE_CHMOD,
     GLOBUS_FTP_CLIENT_FEATURE_AUTHZ_ASSERT,
+    GLOBUS_FTP_CLIENT_FEATURE_CLIENTINFO,
     GLOBUS_FTP_CLIENT_FEATURE_SITE_SETNETSTACK,
     GLOBUS_FTP_CLIENT_FEATURE_DELAYED_PASV,
     GLOBUS_FTP_CLIENT_FEATURE_GETPUT,
