@@ -19,8 +19,8 @@
  * @file globus_ftp_client_handle.c FTP Handle Manipulation.
  *
  * $RCSfile: globus_ftp_client_handle.c,v $
- * $Revision: 1.40 $
- * $Date: 2007/11/09 21:57:44 $
+ * $Revision: 1.45 $
+ * $Date: 2010/03/24 04:50:35 $
  */
 #endif
 
@@ -99,7 +99,7 @@ globus_l_ftp_client_target_delete(
 
 static char *                           globus_l_ftp_client_ssh_client_program = NULL;
 
-void
+char *
 globus_i_ftp_client_find_ssh_client_program()
 {
     char *                              gl;
@@ -112,7 +112,7 @@ globus_i_ftp_client_find_ssh_client_program()
     if(result == GLOBUS_SUCCESS)
     {
         path = globus_common_create_string("%s/.globus/%s",
-            hd, SSH_EXEC_SCRIPT);
+            hd, GLOBUS_L_FTP_CLIENT_SSH_EXEC_SCRIPT);
         free(hd);
         result = GLOBUS_GSI_SYSCONFIG_FILE_EXISTS(path);
         if(result == GLOBUS_SUCCESS)
@@ -132,7 +132,7 @@ globus_i_ftp_client_find_ssh_client_program()
         if(result == GLOBUS_SUCCESS)
         {
             path = globus_common_create_string("%s/libexec/%s",
-                gl, SSH_EXEC_SCRIPT);
+                gl, GLOBUS_L_FTP_CLIENT_SSH_EXEC_SCRIPT);
             free(gl);
             result = GLOBUS_GSI_SYSCONFIG_FILE_EXISTS(path);
             if(result == GLOBUS_SUCCESS)
@@ -150,7 +150,7 @@ globus_i_ftp_client_find_ssh_client_program()
     if(globus_l_ftp_client_ssh_client_program == NULL)
     {
         path = globus_common_create_string(
-            "/etc/grid-security/%s", SSH_EXEC_SCRIPT);
+            "/etc/grid-security/%s", GLOBUS_L_FTP_CLIENT_SSH_EXEC_SCRIPT);
         result = GLOBUS_GSI_SYSCONFIG_FILE_EXISTS(path);
         if(result == GLOBUS_SUCCESS)
         {
@@ -161,6 +161,8 @@ globus_i_ftp_client_find_ssh_client_program()
             free(path);
         }
     }
+    
+    return globus_l_ftp_client_ssh_client_program;
 }
 
 /**
@@ -564,6 +566,44 @@ error:
 /* globus_ftp_client_handle_get_user_pointer() */
 /*@}*/
 
+globus_result_t
+globus_ftp_client_handle_get_restart_marker(
+    const globus_ftp_client_handle_t *          handle,
+    globus_ftp_client_restart_marker_t *        marker)
+{
+    globus_object_t *                           err;
+    globus_i_ftp_client_handle_t *              i_handle;
+    GlobusFuncName(globus_ftp_client_handle_get_restart_marker);
+
+    if(handle == GLOBUS_NULL)
+    {
+        err = GLOBUS_I_FTP_CLIENT_ERROR_NULL_PARAMETER("handle");
+
+        goto error;
+    }
+    else if(marker == GLOBUS_NULL)
+    {
+        err = GLOBUS_I_FTP_CLIENT_ERROR_NULL_PARAMETER("marker");
+
+        goto error;
+    }
+    if(GLOBUS_I_FTP_CLIENT_BAD_MAGIC(handle))
+    {
+        err = GLOBUS_I_FTP_CLIENT_ERROR_INVALID_PARAMETER("handle");
+
+        goto error;
+    }
+    i_handle = *(globus_i_ftp_client_handle_t **) handle;
+    globus_ftp_client_restart_marker_copy(marker, &i_handle->restart_marker);
+
+    return GLOBUS_SUCCESS;
+
+error:
+    return globus_error_put(err);
+}
+/* globus_ftp_client_handle_get_restart_marker() */
+
+
 /**
  * @name Plugins
  */
@@ -1181,6 +1221,10 @@ globus_l_ftp_client_target_delete(
     {
         globus_libc_free(target->auth_info.auth_gssapi_subject);
     }
+    if(target->dcau.subject.subject)
+    {
+        globus_libc_free(target->dcau.subject.subject);
+    }
     if(target->net_stack_str)
     {
         globus_libc_free(target->net_stack_str);
@@ -1193,6 +1237,10 @@ globus_l_ftp_client_target_delete(
     {
         globus_libc_free(target->authz_assert);
     }
+    if(target->clientinfo_argstr)
+    {
+        globus_libc_free(target->clientinfo_argstr);
+    }
     if(target->features)
     {
         globus_i_ftp_client_features_destroy(target->features);
@@ -1341,7 +1389,7 @@ globus_object_t *
 globus_i_ftp_client_target_find(
     globus_i_ftp_client_handle_t *		handle,
     const char *				url,
-    globus_i_ftp_client_operationattr_t *	attr,
+    globus_i_ftp_client_operationattr_t *       attr_in,
     globus_i_ftp_client_target_t **		target)
 {
     globus_url_t				parsed_url;
@@ -1349,6 +1397,8 @@ globus_i_ftp_client_target_find(
     globus_list_t *				node;
     globus_i_ftp_client_cache_entry_t *		cache_entry;
     globus_l_ftp_client_target_search_t         searcher;
+    globus_i_ftp_client_operationattr_t *       attr;
+    globus_result_t                             result;
 
     GlobusFuncName(globus_i_ftp_client_target_find);
 
@@ -1371,6 +1421,20 @@ globus_i_ftp_client_target_find(
 
     (*target) = GLOBUS_NULL;
 
+    if(attr_in)
+    {
+        attr = attr_in;
+    }
+    else
+    {
+        result = globus_ftp_client_operationattr_init(&attr);
+        if(result)
+        {
+            err = globus_error_get(result);
+            goto error_exit;
+        }
+    }
+    
     /* Search the cache for a target which matches this URL. */
     searcher.url = &parsed_url;
     searcher.attr = attr;
@@ -1405,9 +1469,7 @@ globus_i_ftp_client_target_find(
 						   attr);
     }
     else /* found copy in cache... update attrs, url */
-    {
-	globus_result_t				result;
-	
+    {	
 	globus_ftp_client_operationattr_destroy(&(*target)->attr);
 	if(attr)
 	{
@@ -1468,6 +1530,10 @@ globus_i_ftp_client_target_find(
     globus_ftp_control_ipv6_allow(
         (*target)->control_handle, (*target)->attr->allow_ipv6);
     
+    if(!attr_in)
+    {
+        globus_ftp_client_operationattr_destroy(&attr);
+    }
     globus_url_destroy(&parsed_url);
 
     globus_i_ftp_client_control_is_active((*target)->control_handle);
@@ -1490,6 +1556,12 @@ free_target:
     }
 free_url:
     globus_url_destroy(&parsed_url);
+        
+    if(!attr_in)
+    {
+        globus_ftp_client_operationattr_destroy(&attr);
+    }
+
 error_exit:
     globus_i_ftp_client_debug_printf(1, 
         (stderr, "globus_i_ftp_client_target_find() exiting with error\n"));
@@ -1569,20 +1641,20 @@ globus_l_ftp_client_url_parse(
     }
     else if(url->scheme_type == GLOBUS_URL_SCHEME_SSHFTP)
     {
-        if(!ftp_client_i_popen_ready)
-        {
-            err = GLOBUS_I_FTP_CLIENT_ERROR_UNSUPPORTED_FEATURE("popen driver not installed");  
-            return err;
-        }
         if(globus_l_ftp_client_ssh_client_program == NULL)
         {
             err = GLOBUS_I_FTP_CLIENT_ERROR_UNSUPPORTED_FEATURE("SSH client script not installed");
             return err;
         }
-    	if(url->port == 0)
-	    {
-	        url->port = 22;	/* IANA-defined SSH port*/
-	    }
+        if(!ftp_client_i_popen_ready)
+        {
+            err = GLOBUS_I_FTP_CLIENT_ERROR_UNSUPPORTED_FEATURE("popen driver not installed");  
+            return err;
+        }
+        if(url->port == 0)
+        {
+            url->port = 22;	/* IANA-defined SSH port*/
+        }
     }
     else
     {
@@ -1695,7 +1767,7 @@ globus_l_ftp_client_compare_canonically(
        cache_entry->url.port == key->url->port &&
        (strcmp(cache_entry->url.host, key->url->host) == 0))
     {
-        if(cache_entry->target && !key->want_empty)
+        if(cache_entry->target && key->attr && !key->want_empty)
 	{
 	    if(globus_ftp_control_auth_info_compare(
 	           &cache_entry->target->attr->auth_info,
